@@ -1,6 +1,6 @@
 /*  Plzip - Parallel compressor compatible with lzip
     Copyright (C) 2009 Laszlo Ersek.
-    Copyright (C) 2009-2014 Antonio Diaz Diaz.
+    Copyright (C) 2009-2015 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -156,9 +156,11 @@ const char * const mem_msg = "Not enough memory. Try a smaller dictionary size";
 
 struct Packet			// data block with a serial number
   {
-  unsigned id;			// serial number assigned as received
   uint8_t * data;
   int size;			// number of bytes in data (if any)
+  unsigned id;			// serial number assigned as received
+  Packet( uint8_t * const d, const int s, const unsigned i )
+    : data( d ), size( s ), id( i ) {}
   };
 
 
@@ -207,10 +209,7 @@ public:
   // make a packet with data received from splitter
   void receive_packet( uint8_t * const data, const int size )
     {
-    Packet * const ipacket = new Packet;
-    ipacket->id = receive_id++;
-    ipacket->data = data;
-    ipacket->size = size;
+    Packet * const ipacket = new Packet( data, size, receive_id++ );
     slot_tally.get_slot();		// wait for a free slot
     xlock( &imutex );
     packet_queue.push( ipacket );
@@ -252,7 +251,7 @@ public:
     xlock( &omutex );
     // id collision shouldn't happen
     if( circular_buffer[i] != 0 )
-      internal_error( "id collision in collect_packet" );
+      internal_error( "id collision in collect_packet." );
     // merge packet into circular buffer
     circular_buffer[i] = opacket;
     if( opacket->id == deliver_id ) xsignal( &oav_or_exit );
@@ -310,6 +309,7 @@ struct Splitter_arg
   const Pretty_print * pp;
   int infd;
   int data_size;
+  int offset;
   };
 
 
@@ -322,12 +322,13 @@ extern "C" void * csplitter( void * arg )
   const Pretty_print & pp = *tmp.pp;
   const int infd = tmp.infd;
   const int data_size = tmp.data_size;
+  const int offset = tmp.offset;
 
   for( bool first_post = true; ; first_post = false )
     {
-    uint8_t * const data = new( std::nothrow ) uint8_t[data_size];
+    uint8_t * const data = new( std::nothrow ) uint8_t[offset+data_size];
     if( !data ) { pp( mem_msg ); cleanup_and_fail(); }
-    const int size = readblock( infd, data, data_size );
+    const int size = readblock( infd, data + offset, data_size );
     if( size != data_size && errno )
       { pp(); show_error( "Read error", errno ); cleanup_and_fail(); }
 
@@ -354,6 +355,7 @@ struct Worker_arg
   const Pretty_print * pp;
   int dictionary_size;
   int match_len_limit;
+  int offset;
   };
 
 
@@ -366,17 +368,17 @@ extern "C" void * cworker( void * arg )
   const Pretty_print & pp = *tmp.pp;
   const int dictionary_size = tmp.dictionary_size;
   const int match_len_limit = tmp.match_len_limit;
+  const int offset = tmp.offset;
 
   while( true )
     {
     Packet * const packet = courier.distribute_packet();
     if( !packet ) break;		// no more packets to process
 
-    const int max_compr_size = 42 + packet->size + ( ( packet->size + 7 ) / 8 );
-    uint8_t * const new_data = new( std::nothrow ) uint8_t[max_compr_size];
-    if( !new_data ) { pp( mem_msg ); cleanup_and_fail(); }
-    const int dict_size = std::max( LZ_min_dictionary_size(),
-                                    std::min( dictionary_size, packet->size ) );
+    const bool fast = dictionary_size == 65535 && match_len_limit == 16;
+    const int dict_size = fast ? dictionary_size :
+                          std::max( std::min( dictionary_size, packet->size ),
+                                    LZ_min_dictionary_size() );
     LZ_Encoder * const encoder =
       LZ_compress_open( dict_size, match_len_limit, LLONG_MAX );
     if( !encoder || LZ_compress_errno( encoder ) != LZ_ok )
@@ -384,7 +386,7 @@ extern "C" void * cworker( void * arg )
       if( !encoder || LZ_compress_errno( encoder ) == LZ_mem_error )
         pp( mem_msg );
       else
-        internal_error( "invalid argument to encoder" );
+        internal_error( "invalid argument to encoder." );
       cleanup_and_fail();
       }
 
@@ -396,35 +398,34 @@ extern "C" void * cworker( void * arg )
         {
         if( written < packet->size )
           {
-          const int wr = LZ_compress_write( encoder, packet->data + written,
+          const int wr = LZ_compress_write( encoder,
+                                            packet->data + offset + written,
                                             packet->size - written );
-          if( wr < 0 ) internal_error( "library error (LZ_compress_write)" );
+          if( wr < 0 ) internal_error( "library error (LZ_compress_write)." );
           written += wr;
           }
-        if( written >= packet->size )
-          { delete[] packet->data; LZ_compress_finish( encoder ); }
+        if( written >= packet->size ) LZ_compress_finish( encoder );
         }
-      const int rd = LZ_compress_read( encoder, new_data + new_pos,
-                                       max_compr_size - new_pos );
+      const int rd = LZ_compress_read( encoder, packet->data + new_pos,
+                                       offset + written - new_pos );
       if( rd < 0 )
         {
         pp();
         if( verbosity >= 0 )
-          std::fprintf( stderr, "LZ_compress_read error: %s.\n",
+          std::fprintf( stderr, "LZ_compress_read error: %s\n",
                         LZ_strerror( LZ_compress_errno( encoder ) ) );
         cleanup_and_fail();
         }
       new_pos += rd;
-      if( new_pos > max_compr_size )
-        internal_error( "packet size exceeded in worker" );
+      if( new_pos >= offset + written )
+        internal_error( "packet size exceeded in worker." );
       if( LZ_compress_finished( encoder ) == 1 ) break;
       }
 
     if( LZ_compress_close( encoder ) < 0 )
       { pp( "LZ_compress_close failed." ); cleanup_and_fail(); }
 
-    if( verbosity >= 2 && packet->size > 0 ) show_progress( packet->size );
-    packet->data = new_data;
+    if( packet->size > 0 ) show_progress( packet->size );
     packet->size = new_pos;
     courier.collect_packet( packet );
     }
@@ -447,12 +448,9 @@ void muxer( Packet_courier & courier, const Pretty_print & pp, const int outfd )
       const Packet * const opacket = packet_vector[i];
       out_size += opacket->size;
 
-      if( outfd >= 0 )
-        {
-        const int wr = writeblock( outfd, opacket->data, opacket->size );
-        if( wr != opacket->size )
-          { pp(); show_error( "Write error", errno ); cleanup_and_fail(); }
-        }
+      const int wr = writeblock( outfd, opacket->data, opacket->size );
+      if( wr != opacket->size )
+        { pp(); show_error( "Write error", errno ); cleanup_and_fail(); }
       delete[] opacket->data;
       delete opacket;
       }
@@ -469,6 +467,7 @@ int compress( const int data_size, const int dictionary_size,
               const int infd, const int outfd,
               const Pretty_print & pp, const int debug_level )
   {
+  const int offset = data_size / 8;
   const int slots_per_worker = 2;
   const int num_slots =
     ( ( num_workers > 1 ) ? num_workers * slots_per_worker : 1 );
@@ -481,6 +480,7 @@ int compress( const int data_size, const int dictionary_size,
   splitter_arg.pp = &pp;
   splitter_arg.infd = infd;
   splitter_arg.data_size = data_size;
+  splitter_arg.offset = offset;
 
   pthread_t splitter_thread;
   int errcode = pthread_create( &splitter_thread, 0, csplitter, &splitter_arg );
@@ -492,6 +492,7 @@ int compress( const int data_size, const int dictionary_size,
   worker_arg.pp = &pp;
   worker_arg.dictionary_size = dictionary_size;
   worker_arg.match_len_limit = match_len_limit;
+  worker_arg.offset = offset;
 
   pthread_t * worker_threads = new( std::nothrow ) pthread_t[num_workers];
   if( !worker_threads ) { pp( mem_msg ); cleanup_and_fail(); }
@@ -519,7 +520,7 @@ int compress( const int data_size, const int dictionary_size,
   if( verbosity >= 1 )
     {
     if( in_size == 0 || out_size == 0 )
-      std::fprintf( stderr, " no data compressed.\n" );
+      std::fputs( " no data compressed.\n", stderr );
     else
       std::fprintf( stderr, "%6.3f:1, %6.3f bits/byte, "
                             "%5.2f%% saved, %llu in, %llu out.\n",
@@ -540,6 +541,6 @@ int compress( const int data_size, const int dictionary_size,
       courier.ocheck_counter,
       courier.owait_counter );
 
-  if( !courier.finished() ) internal_error( "courier not finished" );
+  if( !courier.finished() ) internal_error( "courier not finished." );
   return 0;
   }
