@@ -1,6 +1,6 @@
 /*  Plzip - Parallel compressor compatible with lzip
     Copyright (C) 2009 Laszlo Ersek.
-    Copyright (C) 2009-2014 Antonio Diaz Diaz.
+    Copyright (C) 2009-2015 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -37,6 +37,23 @@
 #include "file_index.h"
 
 
+void Pretty_print::operator()( const char * const msg ) const
+  {
+  if( verbosity >= 0 )
+    {
+    if( first_post )
+      {
+      first_post = false;
+      std::fprintf( stderr, "  %s: ", name_.c_str() );
+      for( unsigned i = 0; i < longest_name - name_.size(); ++i )
+        std::fputc( ' ', stderr );
+      if( !msg ) std::fflush( stderr );
+      }
+    if( msg ) std::fprintf( stderr, "%s\n", msg );
+    }
+  }
+
+
 // Returns the number of bytes really read.
 // If (returned value < size) and (errno == 0), means EOF was reached.
 //
@@ -82,7 +99,7 @@ int decompress_read_error( struct LZ_Decoder * const decoder,
   const LZ_Errno errcode = LZ_decompress_errno( decoder );
   pp();
   if( verbosity >= 0 )
-    std::fprintf( stderr, "LZ_decompress_read error in worker %d: %s.\n",
+    std::fprintf( stderr, "LZ_decompress_read error in worker %d: %s\n",
                   worker_id, LZ_strerror( errcode ) );
   if( errcode == LZ_header_error || errcode == LZ_unexpected_eof ||
       errcode == LZ_data_error )
@@ -160,7 +177,7 @@ extern "C" void * dworker( void * arg )
             {
             pp();
             if( verbosity >= 0 )
-              std::fprintf( stderr, "Write error in worker %d: %s.\n",
+              std::fprintf( stderr, "Write error in worker %d: %s\n",
                             worker_id, std::strerror( errno ) );
             cleanup_and_fail();
             }
@@ -196,20 +213,21 @@ extern "C" void * dworker( void * arg )
     // start the workers and wait for them to finish.
 int decompress( int num_workers, const int infd, const int outfd,
                 const Pretty_print & pp, const int debug_level,
-                const bool testing, const bool infd_isreg )
+                const bool infd_isreg )
   {
   if( !infd_isreg )
-    return dec_stream( num_workers, infd, outfd, pp, debug_level, testing );
+    return dec_stream( num_workers, infd, outfd, pp, debug_level );
 
   const File_index file_index( infd );
   if( file_index.retval() == 1 )
     {
     lseek( infd, 0, SEEK_SET );
-    return dec_stream( num_workers, infd, outfd, pp, debug_level, testing );
+    return dec_stream( num_workers, infd, outfd, pp, debug_level );
     }
   if( file_index.retval() != 0 )
     { pp( file_index.error().c_str() ); return file_index.retval(); }
 
+  show_header( file_index.dictionary_size( 0 ) );
   if( num_workers > file_index.members() )
     num_workers = file_index.members();
 
@@ -255,11 +273,11 @@ int decompress( int num_workers, const int infd, const int outfd,
                   (double)out_size / in_size,
                   ( 8.0 * in_size ) / out_size,
                   100.0 * ( 1.0 - ( (double)in_size / out_size ) ) );
-  if( verbosity >= 3 )
+  if( verbosity >= 4 )
     std::fprintf( stderr, "decompressed size %9llu, size %9llu.  ",
                   out_size, in_size );
 
-  if( verbosity >= 1 ) std::fprintf( stderr, testing ? "ok\n" : "done\n" );
+  if( verbosity >= 1 ) std::fputs( (outfd < 0) ? "ok\n" : "done\n", stderr );
 
   return 0;
   }
