@@ -1,6 +1,6 @@
 /*  Plzip - Parallel compressor compatible with lzip
     Copyright (C) 2009 Laszlo Ersek.
-    Copyright (C) 2009-2014 Antonio Diaz Diaz.
+    Copyright (C) 2009-2015 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -72,7 +72,7 @@ namespace {
 
 const char * const Program_name = "Plzip";
 const char * const program_name = "plzip";
-const char * const program_year = "2014";
+const char * const program_year = "2015";
 const char * invocation_name = 0;
 
 struct { const char * from; const char * to; } const known_extensions[] = {
@@ -103,11 +103,11 @@ void show_help( const long num_online )
   std::printf( "\nOptions:\n"
                "  -h, --help                     display this help and exit\n"
                "  -V, --version                  output version information and exit\n"
-               "  -B, --data-size=<bytes>        set size of input data blocks, in bytes\n"
+               "  -B, --data-size=<bytes>        set size of input data blocks [2x8=16 MiB]\n"
                "  -c, --stdout                   send output to standard output\n"
                "  -d, --decompress               decompress\n"
                "  -f, --force                    overwrite existing output files\n"
-               "  -F, --recompress               force recompression of compressed files\n"
+               "  -F, --recompress               force re-compression of compressed files\n"
                "  -k, --keep                     keep (don't delete) input files\n"
                "  -m, --match-length=<bytes>     set match length limit in bytes [36]\n"
                "  -n, --threads=<n>              set number of (de)compression threads [%ld]\n"
@@ -116,10 +116,10 @@ void show_help( const long num_online )
                "  -s, --dictionary-size=<bytes>  set dictionary size limit in bytes [8 MiB]\n"
                "  -t, --test                     test compressed file integrity\n"
                "  -v, --verbose                  be verbose (a 2nd -v gives more)\n"
-               "  -1 .. -9                       set compression level [default 6]\n"
-               "      --fast                     alias for -1\n"
+               "  -0 .. -9                       set compression level [default 6]\n"
+               "      --fast                     alias for -0\n"
                "      --best                     alias for -9\n", num_online );
-  if( verbosity > 0 )
+  if( verbosity >= 1 )
     {
     std::printf( "  -D, --debug=<level>        (0-1) print debug statistics to stderr\n" );
     }
@@ -130,8 +130,7 @@ void show_help( const long num_online )
                "The bidimensional parameter space of LZMA can't be mapped to a linear\n"
                "scale optimal for all files. If your files are large, very repetitive,\n"
                "etc, you may need to use the --match-length and --dictionary-size\n"
-               "options directly to achieve optimal performance. For example, -9m64\n"
-               "usually compresses executables more (and faster) than -9.\n"
+               "options directly to achieve optimal performance.\n"
                "\nExit status: 0 for a normal exit, 1 for environmental problems (file\n"
                "not found, invalid flags, I/O errors, etc), 2 to indicate a corrupt or\n"
                "invalid input file, 3 for an internal consistency error (eg, bug) which\n"
@@ -146,12 +145,34 @@ void show_version()
   std::printf( "%s %s\n", program_name, PROGVERSION );
   std::printf( "Copyright (C) 2009 Laszlo Ersek.\n"
                "Copyright (C) %s Antonio Diaz Diaz.\n", program_year );
-  std::printf( "Using Lzlib %s\n", LZ_version() );
+  std::printf( "Using lzlib %s\n", LZ_version() );
   std::printf( "License GPLv2+: GNU GPL version 2 or later <http://gnu.org/licenses/gpl.html>\n"
                "This is free software: you are free to change and redistribute it.\n"
                "There is NO WARRANTY, to the extent permitted by law.\n" );
   }
 
+} // end namespace
+
+void show_header( const unsigned dictionary_size )
+  {
+  if( verbosity >= 3 )
+    {
+    const char * const prefix[8] =
+      { "Ki", "Mi", "Gi", "Ti", "Pi", "Ei", "Zi", "Yi" };
+    enum { factor = 1024 };
+    const char * p = "";
+    const char * np = "  ";
+    unsigned num = dictionary_size;
+    bool exact = ( num % factor == 0 );
+
+    for( int i = 0; i < 8 && ( num > 9999 || ( exact && num >= factor ) ); ++i )
+      { num /= factor; if( num % factor != 0 ) exact = false;
+        p = prefix[i]; np = ""; }
+    std::fprintf( stderr, "dictionary size %s%4u %sB.  ", np, num, p );
+    }
+  }
+
+namespace {
 
 unsigned long long getnum( const char * const ptr,
                            const unsigned long long llimit,
@@ -168,7 +189,7 @@ unsigned long long getnum( const char * const ptr,
 
   if( !errno && tail[0] )
     {
-    int factor = ( tail[1] == 'i' ) ? 1024 : 1000;
+    const int factor = ( tail[1] == 'i' ) ? 1024 : 1000;
     int exponent = 0;
     bool bad_multiplier = false;
     switch( tail[0] )
@@ -211,11 +232,14 @@ unsigned long long getnum( const char * const ptr,
 int get_dict_size( const char * const arg )
   {
   char * tail;
-  int bits = std::strtol( arg, &tail, 0 );
+  const int bits = std::strtol( arg, &tail, 0 );
   if( bits >= LZ_min_dictionary_bits() &&
       bits <= LZ_max_dictionary_bits() && *tail == 0 )
     return ( 1 << bits );
-  return getnum( arg, LZ_min_dictionary_size(), LZ_max_dictionary_size() );
+  int dictionary_size = getnum( arg, LZ_min_dictionary_size(),
+                                     LZ_max_dictionary_size() );
+  if( dictionary_size == 65535 ) ++dictionary_size;
+  return dictionary_size;
   }
 
 
@@ -249,7 +273,7 @@ int open_instream( const char * const name, struct stat * const in_statsp,
     if( infd < 0 )
       {
       if( verbosity >= 0 )
-        std::fprintf( stderr, "%s: Can't open input file '%s': %s.\n",
+        std::fprintf( stderr, "%s: Can't open input file '%s': %s\n",
                       program_name, name, std::strerror( errno ) );
       }
     else
@@ -266,7 +290,7 @@ int open_instream( const char * const name, struct stat * const in_statsp,
           std::fprintf( stderr, "%s: Input file '%s' is not a regular file%s.\n",
                         program_name, name,
                         ( can_read && !no_ofile ) ?
-                        " and '--stdout' was not specified" : "" );
+                        ",\n       and '--stdout' was not specified" : "" );
         close( infd );
         infd = -1;
         }
@@ -297,7 +321,7 @@ void set_d_outname( const std::string & name, const int i )
     }
   output_filename = name; output_filename += ".out";
   if( verbosity >= 1 )
-    std::fprintf( stderr, "%s: Can't guess original name for '%s' -- using '%s'.\n",
+    std::fprintf( stderr, "%s: Can't guess original name for '%s' -- using '%s'\n",
                   program_name, name.c_str(), output_filename.c_str() );
   }
 
@@ -314,7 +338,7 @@ bool open_outstream( const bool force )
       std::fprintf( stderr, "%s: Output file '%s' already exists, skipping.\n",
                     program_name, output_filename.c_str() );
     else
-      std::fprintf( stderr, "%s: Can't create output file '%s': %s.\n",
+      std::fprintf( stderr, "%s: Can't create output file '%s': %s\n",
                     program_name, output_filename.c_str(), std::strerror( errno ) );
     }
   return ( outfd >= 0 );
@@ -323,7 +347,7 @@ bool open_outstream( const bool force )
 
 bool check_tty( const int infd, const Mode program_mode )
   {
-  if( program_mode == m_compress && outfd >= 0 && isatty( outfd ) )
+  if( program_mode == m_compress && isatty( outfd ) )
     {
     show_error( "I won't write compressed data to a terminal.", 0, true );
     return false;
@@ -337,6 +361,32 @@ bool check_tty( const int infd, const Mode program_mode )
   return true;
   }
 
+} // end namespace
+
+// This can be called from any thread, main thread or sub-threads alike,
+// since they all call common helper functions that call cleanup_and_fail()
+// in case of an error.
+//
+void cleanup_and_fail( const int retval )
+  {
+  // only one thread can delete and exit
+  static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
+
+  pthread_mutex_lock( &mutex );		// ignore errors to avoid loop
+  if( delete_output_on_interrupt )
+    {
+    delete_output_on_interrupt = false;
+    if( verbosity >= 0 )
+      std::fprintf( stderr, "%s: Deleting output file '%s', if it exists.\n",
+                    program_name, output_filename.c_str() );
+    if( outfd >= 0 ) { close( outfd ); outfd = -1; }
+    if( std::remove( output_filename.c_str() ) != 0 && errno != ENOENT )
+      show_error( "WARNING: deletion of output file (apparently) failed." );
+    }
+  std::exit( retval );
+  }
+
+namespace {
 
      // Set permissions, owner and times.
 void close_and_set_permissions( const struct stat * const in_statsp )
@@ -388,23 +438,6 @@ void set_signals()
 int verbosity = 0;
 
 
-void Pretty_print::operator()( const char * const msg ) const
-  {
-  if( verbosity >= 0 )
-    {
-    if( first_post )
-      {
-      first_post = false;
-      std::fprintf( stderr, "  %s: ", name_.c_str() );
-      for( unsigned i = 0; i < longest_name - name_.size(); ++i )
-        std::fprintf( stderr, " " );
-      if( !msg ) std::fflush( stderr );
-      }
-    if( msg ) std::fprintf( stderr, "%s\n", msg );
-    }
-  }
-
-
 void show_error( const char * const msg, const int errcode, const bool help )
   {
   if( verbosity >= 0 )
@@ -413,8 +446,8 @@ void show_error( const char * const msg, const int errcode, const bool help )
       {
       std::fprintf( stderr, "%s: %s", program_name, msg );
       if( errcode > 0 )
-        std::fprintf( stderr, ": %s.", std::strerror( errcode ) );
-      std::fprintf( stderr, "\n" );
+        std::fprintf( stderr, ": %s", std::strerror( errcode ) );
+      std::fputc( '\n', stderr );
       }
     if( help )
       std::fprintf( stderr, "Try '%s --help' for more information.\n",
@@ -431,30 +464,6 @@ void internal_error( const char * const msg )
   }
 
 
-// This can be called from any thread, main thread or sub-threads alike,
-// since they all call common helper functions that call cleanup_and_fail()
-// in case of an error.
-//
-void cleanup_and_fail( const int retval )
-  {
-  // only one thread can delete and exit
-  static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
-
-  pthread_mutex_lock( &mutex );		// ignore errors to avoid loop
-  if( delete_output_on_interrupt )
-    {
-    delete_output_on_interrupt = false;
-    if( verbosity >= 0 )
-      std::fprintf( stderr, "%s: Deleting output file '%s', if it exists.\n",
-                    program_name, output_filename.c_str() );
-    if( outfd >= 0 ) { close( outfd ); outfd = -1; }
-    if( std::remove( output_filename.c_str() ) != 0 && errno != ENOENT )
-      show_error( "WARNING: deletion of output file (apparently) failed." );
-    }
-  std::exit( retval );
-  }
-
-
 void show_progress( const int packet_size,
                     const Pretty_print * const p,
                     const unsigned long long cfile_size )
@@ -464,17 +473,20 @@ void show_progress( const int packet_size,
   static const Pretty_print * pp = 0;
   static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
 
-  if( p )					// initialize static vars
-    { csize = cfile_size; pos = 0; pp = p; }
-  if( pp )
+  if( verbosity >= 2 )
     {
-    xlock( &mutex );
-    pos += packet_size;
-    if( csize > 0 )
-      std::fprintf( stderr, "%4llu%%", pos / csize );
-    std::fprintf( stderr, "  %.1f MB\r", pos / 1000000.0 );
-    pp->reset(); (*pp)();			// restore cursor position
-    xunlock( &mutex );
+    if( p )					// initialize static vars
+      { csize = cfile_size; pos = 0; pp = p; }
+    if( pp )
+      {
+      xlock( &mutex );
+      pos += packet_size;
+      if( csize > 0 )
+        std::fprintf( stderr, "%4llu%%", pos / csize );
+      std::fprintf( stderr, "  %.1f MB\r", pos / 1000000.0 );
+      pp->reset(); (*pp)();			// restore cursor position
+      xunlock( &mutex );
+      }
     }
   }
 
@@ -485,7 +497,7 @@ int main( const int argc, const char * const argv[] )
      to the corresponding LZMA compression modes. */
   const Lzma_options option_mapping[] =
     {
-    { 1 << 20,   5 },		// -0
+    {   65535,  16 },		// -0
     { 1 << 20,   5 },		// -1
     { 3 << 19,   6 },		// -2
     { 1 << 21,   8 },		// -3
@@ -559,29 +571,29 @@ int main( const int argc, const char * const argv[] )
     {
     const int code = parser.code( argind );
     if( !code ) break;					// no more options
-    const char * const arg = parser.argument( argind ).c_str();
+    const std::string & arg = parser.argument( argind );
     switch( code )
       {
       case '0': case '1': case '2': case '3': case '4':
       case '5': case '6': case '7': case '8': case '9':
                 encoder_options = option_mapping[code-'0']; break;
       case 'b': break;
-      case 'B': data_size = getnum( arg, 2 * LZ_min_dictionary_size(),
+      case 'B': data_size = getnum( arg.c_str(), 2 * LZ_min_dictionary_size(),
                                     2 * LZ_max_dictionary_size() ); break;
       case 'c': to_stdout = true; break;
       case 'd': program_mode = m_decompress; break;
-      case 'D': debug_level = getnum( arg, 0, 3 ); break;
+      case 'D': debug_level = getnum( arg.c_str(), 0, 3 ); break;
       case 'f': force = true; break;
       case 'F': recompress = true; break;
       case 'h': show_help( num_online ); return 0;
       case 'k': keep_input_files = true; break;
       case 'm': encoder_options.match_len_limit =
-                  getnum( arg, LZ_min_match_len_limit(),
-                               LZ_max_match_len_limit() ); break;
-      case 'n': num_workers = getnum( arg, 1, max_workers ); break;
+                  getnum( arg.c_str(), LZ_min_match_len_limit(),
+                                       LZ_max_match_len_limit() ); break;
+      case 'n': num_workers = getnum( arg.c_str(), 1, max_workers ); break;
       case 'o': default_output_filename = arg; break;
       case 'q': verbosity = -1; break;
-      case 's': encoder_options.dictionary_size = get_dict_size( arg );
+      case 's': encoder_options.dictionary_size = get_dict_size( arg.c_str() );
                 break;
       case 'S': break;
       case 't': program_mode = m_test; break;
@@ -599,9 +611,14 @@ int main( const int argc, const char * const argv[] )
   if( program_mode == m_test )
     outfd = -1;
 
+  const bool fast = encoder_options.dictionary_size == 65535 &&
+                    encoder_options.match_len_limit == 16;
   if( data_size <= 0 )
-    data_size = 2 * std::max( 65536, encoder_options.dictionary_size );
-  else if( data_size < encoder_options.dictionary_size )
+    {
+    if( fast ) data_size = 1 << 20;
+    else data_size = 2 * std::max( 65536, encoder_options.dictionary_size );
+    }
+  else if( !fast && data_size < encoder_options.dictionary_size )
     encoder_options.dictionary_size =
       std::max( data_size, LZ_min_dictionary_size() );
 
@@ -688,15 +705,13 @@ int main( const int argc, const char * const argv[] )
     int tmp;
     if( program_mode == m_compress )
       {
-      if( verbosity >= 2 )	// init
-        show_progress( 0, &pp, infd_isreg ? in_statsp->st_size / 100 : 0 );
+      show_progress( 0, &pp, infd_isreg ? in_statsp->st_size / 100 : 0 ); // init
       tmp = compress( data_size, encoder_options.dictionary_size,
                       encoder_options.match_len_limit,
                       num_workers, infd, outfd, pp, debug_level );
       }
     else
-      tmp = decompress( num_workers, infd, outfd, pp, debug_level,
-                        program_mode == m_test, infd_isreg );
+      tmp = decompress( num_workers, infd, outfd, pp, debug_level, infd_isreg );
     if( tmp > retval ) retval = tmp;
     if( tmp && program_mode != m_test ) cleanup_and_fail( retval );
 
